#!/usr/bin/python

from morpho import io, evaluation
from tabulate import tabulate
from optparse import OptionParser
import sys


TABLE_FMT = 'orgtbl'
FLOAT_FMT = '.2f'

encoding = 'utf-8'

def read_simple_segmentation_file(file_name):
    with open(file_name, 'r') as fl:
        lines = fl.readlines()
        lines = [l.decode(encoding).strip().split('+-/ ') for l in lines]
        return lines

def unicodize(lst):
    return [[unicode(el) for el in row] for row in lst]

def utabulate(lst, headers=None, tablefmt=TABLE_FMT):
    if headers:
        return tabulate(unicodize(lst), headers=headers, tablefmt=tablefmt, floatfmt=FLOAT_FMT)
    else:
        return tabulate(unicodize(lst), tablefmt=tablefmt, floatfmt=FLOAT_FMT)


if __name__ == '__main__':
    parser = OptionParser(usage='%prog [options] segmentation_files')
    parser.add_option('-g', '--gold-file', dest='gold_file', help='path to gold standard file.', metavar='FILE')
    parser.add_option('-l', '--label-file', dest='label_file', help='path to label definition file.', metavar='FILE')

    parser.add_option('-v', '--verbose', action='store_true', dest='verbose', default=False, help='print extra information.')
    
    (options, args) = parser.parse_args()
    
    if len(args) == 0:
        print >> sys.stderr, '\nPlease provide a segmentation file name to be evaluated.'
        parser.print_help()
        exit()
    if options.gold_file is None or options.label_file is None:
        print >> sys.stderr, '\nPlease provide gold standard and label definition file names.'
        parser.print_help()
        exit()
        
    seg_file = args[0]
    gold_file  = options.gold_file
    label_file = options.label_file
    evaluation.verbose = options.verbose

    if evaluation.verbose:
        print 'Using verbose'

    gold, labels = evaluation.read_and_validate_gold(gold_file, label_file)
    print
    for seg_file in args:        
        segs = read_simple_segmentation_file(seg_file)

        stats, results = evaluation.evaluate(gold, labels, segs, False)
        print '*************************************************'
        print seg_file
        print utabulate([stats], headers=['#gold bounaries', '#response boundaries', '#intersect(gold, response)', '#matches', '#possible boundaries'])
        print
        print utabulate([results], headers=['precision', 'recall', 'f1', 'acuuracy'])
        print
    

