"""Utilities for reading and writing lexicon and/or segmentations. Probably you need to use model module to create a model after reading in the segmentation files.
"""
from operator import itemgetter
import re
import random
import numpy as np

def process_seg(seg):
    ""
    return [seg[i] if i % 2 == 0 else int(seg[i]) for i in range(len(seg))]

def read_segmentation_file(segFileName):
    segmentationsP = [l.decode('utf-8').strip().split('\t')[0].strip().split() for l in open(segFileName, 'r').readlines()]
    segmentations = [process_seg(l) for l in segmentationsP]
    return segmentations

def read_lex_file(lexFileName, old=False):
    "Reads in a lexicon file and returns a lexicon object as well as number of classes used"
    splits = re.split('(Class [0-9]+)', open(lexFileName, 'r').read().decode('utf-8'))
    classPattern = re.compile(u'^Class [0-9]+$')
    cls = [[splits[i], splits[i+1]] for i in range(1, len(splits), 2)] 
    for c in cls:
        assert re.match(classPattern, c[0])
    cls = sorted([[int(c[0].replace('Class ', '')), [x.split() for x in c[1].split('\n') if len(x)]] for c in cls])
    
    lex = [{morphs[0]:float(morphs[1]) for morphs in c[1]} for c in cls]
    lex_comp = None
    try:
        lex_comp = [{morphs[0]:float(morphs[2]) for morphs in c[1]} for c in cls]
    except:
        lex_comp = None
    if old:
        lex = [{}] + lex + [{}]
        lex_comp = [{}] + lex_comp + [{}]
    return [lex, len(lex) - 2, lex_comp]

    
def print_lex(lex):
    "Print lexicon in a user friendly way"
    classCounter = 0
    for cl in lex:
        print 'Class', classCounter
        li = sorted(sorted([[key, value] for (key, value) in cl.items()], key=itemgetter(0)), key=itemgetter(1), reverse=True)
        print '\n'.join(['%s %.1f'% (l[0].encode('utf-8'),l[1]) for l in li])
        if len(cl) > 0:
            print 
        classCounter += 1

def print_transition_org(transitions):
    print '\n'.join(['|' + '|'.join([str(x) for x in l]) + '|' for l in transitions])

def print_transitions(transitions):
    return np.array(transitions)
    #return  '\n'.join(['|' + '|'.join([str(x) for x in l]) + '|' for l in transitions])

def print_segmentations(segs):
    return '\n'.join([print_word_segmentation(seg) for seg in segs])
        
def print_word_segmentation(seg):
    return u' '.join([unicode(x) for x in seg])
